@@ -1,8 +1,6 @@
 import warnings
 
-from chempy.units import allclose
-from ..water_diffusivity_holz_2000 import water_self_diffusion_coefficient as \
-    w_sd
+from ..water_diffusivity_holz_2000 import water_self_diffusion_coefficient as w_sd
 
 
 def test_water_self_diffusion_coefficient():
@@ -25,12 +23,12 @@ def test_water_self_diffusion_coefficient():
     warnings.resetwarnings()
 
     try:
-        import quantities as pq
-        import numpy as np
-        unit = pq.m**2/pq.s
-        assert allclose(1e9*w_sd(298.15*pq.K, units=pq),
-                        2.299*unit, rtol=1e-3, atol=1e-8*unit)
-        assert allclose(1e9*w_sd(np.linspace(297, 299)*pq.K, units=pq),
-                        2.299*pq.m**2/pq.s, rtol=5e-2, atol=1e-2*unit)
+        from chempy.units import allclose, linspace, default_units as u
     except ImportError:
         pass
+    else:
+        unit = u.m**2/u.s
+        assert allclose(1e9*w_sd(298.15*u.K, units=u),
+                        2.299*unit, rtol=1e-3, atol=1e-8*unit)
+        assert allclose(1e9*w_sd(linspace(297, 299)*u.K, units=u),
+                        2.299*u.m**2/u.s, rtol=5e-2, atol=1e-2*unit)
