@@ -8,8 +8,8 @@ import shutil
 import tempfile
 
 
-def rsys2dot(rsys, tex=False, rprefix='r', rref0=1,
-             nodeparams='[label={} shape=diamond]'):
+def rsys2dot(rsys, tex=False, rprefix='r', rref0=1, nodeparams='[label="{}",shape=diamond]',
+             colors=('maroon', 'darkgreen'), penwidths=None, include_inactive=True):
     """
     Returns list of lines of DOT (graph description language)
     formated graph.
@@ -32,35 +32,59 @@ def rsys2dot(rsys, tex=False, rprefix='r', rref0=1,
     list of lines of DOT representation of the graph representation.
 
     """
-    lines = ['digraph ' + str(rsys.name) + '{\n']
+    lines = ['digraph "' + str(rsys.name) + '" {\n']
     ind = '  '  # indentation
+    if penwidths is None:
+        penwidths = [1.0]*rsys.nr
 
-    def add_vertex(key, num, reac):
+    categories = rsys.categorize_substances(checks=())
+
+    def add_substance(key):
+        fc = 'black'
+        if key in categories['depleted']:
+            fc = colors[0]
+        if key in categories['accumulated']:
+            fc = colors[1]
+        label = ('$%s$' if tex else '%s') % getattr(rsys.substances[key], 'latex_name' if tex else 'name')
+        lines.append(ind + '"{key}" [fontcolor={fc} label="{lbl}"];\n'.format(key=key, fc=fc, lbl=label))
+
+    for sk in rsys.substances:
+        add_substance(sk)
+
+    def add_vertex(key, num, reac, penwidth):
         snum = str(num) if num > 1 else ''
-        name = getattr(rsys.substances[key], 'latex_name' if tex else 'name')
-        lines.append(ind + '"{}" -> "{}" [label ="{}"];\n'.format(
-            *((name, rid, snum) if reac else (rid, name, snum))
+        fmt = ','.join(
+            ['label="{}"'.format(snum)] +
+            (['penwidth={}'.format(penwidth)] if penwidth != 1 else [])
+        )
+        lines.append(ind + '"{}" -> "{}" [color={},fontcolor={},{}];\n'.format(
+            *((key, rid, colors[0], colors[0], fmt) if reac else
+              (rid, key, colors[1], colors[1], fmt))
         ))
 
-    all_reac_stoichs = rsys.all_reac_stoichs()
-    all_prod_stoichs = rsys.all_prod_stoichs()
+    if include_inactive:
+        reac_stoichs = rsys.all_reac_stoichs()
+        prod_stoichs = rsys.all_prod_stoichs()
+    else:
+        reac_stoichs = rsys.active_reac_stoichs()
+        prod_stoichs = rsys.active_prod_stoichs()
 
     for ri, rxn in enumerate(rsys.rxns):
         rid = rprefix + str(ri+rref0)
         lines.append(ind + '{')
-        lines.append(ind*2 + 'node ' + nodeparams.format(rid))
+        lines.append(ind*2 + 'node ' + nodeparams.format(rxn.name or rid))
         lines.append(ind*2 + rid)
         lines.append(ind + '}\n')
         for idx, key in enumerate(rsys.substances):
-            num = all_reac_stoichs[ri, idx]
+            num = reac_stoichs[ri, idx]
             if num == 0:
                 continue
-            add_vertex(key, num, True)
+            add_vertex(key, num, True, penwidths[ri])
         for idx, key in enumerate(rsys.substances):
-            num = all_prod_stoichs[ri, idx]
+            num = prod_stoichs[ri, idx]
             if num == 0:
                 continue
-            add_vertex(key, num, False)
+            add_vertex(key, num, False, penwidths[ri])
     lines.append('}\n')
     return lines
 
@@ -72,17 +96,17 @@ def rsys2graph(rsys, fname, output_dir=None, prog=None, save=False, **kwargs):
 
     Parameters
     ----------
-    rsys: ReactionSystem
-    fname: str
+    rsys : ReactionSystem
+    fname : str
         filename
-    output_dir: str (optional)
+    output_dir : str (optional)
         path to directory (default: temporary directory)
-    prog: str (optional)
+    prog : str (optional)
         default: 'dot'
-    save: bool
+    save : bool
         removes temporary directory if False, default: False
-    \*\*kwargs:
-        parameters to pass along to `rsys2dot`
+    \\*\\*kwargs :
+        Keyword arguments passed along to py:func:`rsys2dot`.
 
     Returns
     -------
