@@ -8,14 +8,34 @@ from collections import defaultdict
 import re
 import warnings
 
-from pyparsing import (
-    Forward, Group, OneOrMore, Optional, ParseResults,
-    Regex, Suppress, Word, nums
-)
+from .pyutil import ChemPyDeprecationWarning, memoize
+from .periodic import symbols
 
-from .pyutil import ChemPyDeprecationWarning
+parsing_library = 'pyparsing'  # info used for selective testing.
 
 
+def get_parsing_context():
+    """ returns the default dictionary for parsing strings in chempy """
+    import chempy
+    from chempy.kinetics import rates
+    from chempy.units import default_units, default_constants, to_unitless
+    import numpy
+    globals_ = dict(to_unitless=to_unitless, chempy=chempy)
+
+    def _update(mod, keys=None):
+        if keys is None:
+            keys = dir(mod)
+        globals_.update({k: getattr(mod, k) for k in keys if not k.startswith('_')})
+    _update(numpy, keys='array log exp'.split())  # could of course add add more
+    _update(rates)
+    _update(chempy)
+    for df in [default_units, default_constants]:
+        if df is not None:
+            globals_.update(df.as_dict())
+    return globals_
+
+
+@memoize()
 def _get_formula_parser():
     """ Create a forward pyparsing parser for chemical formulae
 
@@ -46,6 +66,10 @@ def _get_formula_parser():
         (see: http://blog.stackoverflow.com/2009/06/attribution-required/)
 
     """
+    _p = __import__(parsing_library)
+    Forward, Group, OneOrMore = _p.Forward, _p.Group, _p.OneOrMore
+    Optional, ParseResults, Regex = _p.Optional, _p.ParseResults, _p.Regex
+    Suppress, Word, nums = _p.Suppress, _p.Word, _p.nums
 
     LPAR, RPAR = map(Suppress, "()")
     integer = Word(nums)
@@ -68,9 +92,6 @@ def _get_formula_parser():
     term = Group((element | Group(LPAR + formula + RPAR)("subgroup")) +
                  Optional(integer, default=1)("mult"))
 
-    # define contents of a formula as one or more terms
-    formula << OneOrMore(term)
-
     # add parse actions for parse-time processing
 
     # parse action to multiply out subgroups
@@ -99,125 +120,12 @@ def _get_formula_parser():
             for t in tokens:
                 ctr[t[0]] += t[1]
             return ParseResults([ParseResults([k, v]) for k, v in ctr.items()])
+    # define contents of a formula as one or more terms
+    formula << OneOrMore(term)
     formula.setParseAction(sumByElement)
 
     return formula
 
-_parser = _get_formula_parser()
-
-symbols = (
-    'H', 'He', 'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na', 'Mg', 'Al',
-    'Si', 'P', 'S', 'Cl', 'Ar', 'K', 'Ca', 'Sc', 'Ti', 'V', 'Cr', 'Mn', 'Fe',
-    'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr',
-    'Y', 'Zr', 'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn',
-    'Sb', 'Te', 'I', 'Xe', 'Cs', 'Ba', 'La', 'Ce', 'Pr', 'Nd', 'Pm', 'Sm',
-    'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu', 'Hf', 'Ta', 'W',
-    'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 'Bi', 'Po', 'At', 'Rn',
-    'Fr', 'Ra', 'Ac', 'Th', 'Pa', 'U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf',
-    'Es', 'Fm', 'Md', 'No', 'Lr', 'Rf', 'Db', 'Sg', 'Bh', 'Hs', 'Mt', 'Ds',
-    'Rg', 'Cn', 'Uut', 'Fl', 'Uup', 'Lv', 'Uus', 'Uuo'
-)
-
-names = (
-    'Hydrogen', 'Helium', 'Lithium', 'Beryllium',
-    'Boron', 'Carbon', 'Nitrogen', 'Oxygen', 'Fluorine', 'Neon', 'Sodium',
-    'Magnesium', 'Aluminium', 'Silicon', 'Phosphorus', 'Sulfur',
-    'Chlorine', 'Argon', 'Potassium', 'Calcium', 'Scandium', 'Titanium',
-    'Vanadium', 'Chromium', 'Manganese', 'Iron', 'Cobalt', 'Nickel',
-    'Copper', 'Zinc', 'Gallium', 'Germanium', 'Arsenic', 'Selenium',
-    'Bromine', 'Krypton', 'Rubidium', 'Strontium', 'Yttrium', 'Zirconium',
-    'Niobium', 'Molybdenum', 'Technetium', 'Ruthenium', 'Rhodium',
-    'Palladium', 'Silver', 'Cadmium', 'Indium', 'Tin', 'Antimony',
-    'Tellurium', 'Iodine', 'Xenon', 'Caesium', 'Barium', 'Lanthanum',
-    'Cerium', 'Praseodymium', 'Neodymium', 'Promethium', 'Samarium',
-    'Europium', 'Gadolinium', 'Terbium', 'Dysprosium', 'Holmium',
-    'Erbium', 'Thulium', 'Ytterbium', 'Lutetium', 'Hafnium', 'Tantalum',
-    'Tungsten', 'Rhenium', 'Osmium', 'Iridium', 'Platinum', 'Gold',
-    'Mercury', 'Thallium', 'Lead', 'Bismuth', 'Polonium', 'Astatine',
-    'Radon', 'Francium', 'Radium', 'Actinium', 'Thorium', 'Protactinium',
-    'Uranium', 'Neptunium', 'Plutonium', 'Americium', 'Curium',
-    'Berkelium', 'Californium', 'Einsteinium', 'Fermium', 'Mendelevium',
-    'Nobelium', 'Lawrencium', 'Rutherfordium', 'Dubnium', 'Seaborgium',
-    'Bohrium', 'Hassium', 'Meitnerium', 'Darmstadtium', 'Roentgenium',
-    'Copernicium', '(Ununtrium)', 'Flerovium', '(Ununpentium)',
-    'Livermorium', '(Ununseptium)', '(Ununoctium)'
-)
-
-lower_names = tuple(n.lower().lstrip('(').rstrip(')') for n in names)
-
-
-def atomic_number(name):
-    try:
-        return symbols.index(name) + 1
-    except ValueError:
-        return lower_names.index(name.lower()) + 1
-
-# The data in '_relative_atomic_masses' is licensed under the CC-SA license
-# https://en.wikipedia.org/w/index.php?title=List_of_elements&oldid=700476748
-_relative_atomic_masses = (
-    "1.008 4.002602(2) 6.94 9.0121831(5) 10.81 12.011 14.007 15.999"
-    " 18.998403163(6) 20.1797(6) 22.98976928(2) 24.305 26.9815385(7) 28.085"
-    " 30.973761998(5) 32.06 35.45 39.948(1) 39.0983(1) 40.078(4)"
-    " 44.955908(5) 47.867(1) 50.9415(1) 51.9961(6) 54.938044(3) 55.845(2)"
-    " 58.933194(4) 58.6934(4) 63.546(3) 65.38(2) 69.723(1) 72.630(8)"
-    " 74.921595(6) 78.971(8) 79.904 83.798(2) 85.4678(3) 87.62(1)"
-    " 88.90584(2) 91.224(2) 92.90637(2) 95.95(1) [98] 101.07(2) 102.90550(2)"
-    " 106.42(1) 107.8682(2) 112.414(4) 114.818(1) 118.710(7) 121.760(1)"
-    " 127.60(3) 126.90447(3) 131.293(6) 132.90545196(6) 137.327(7)"
-    " 138.90547(7) 140.116(1) 140.90766(2) 144.242(3) [145] 150.36(2)"
-    " 151.964(1) 157.25(3) 158.92535(2) 162.500(1) 164.93033(2) 167.259(3)"
-    " 168.93422(2) 173.045(10) 174.9668(1) 178.49(2) 180.94788(2) 183.84(1)"
-    " 186.207(1) 190.23(3) 192.217(3) 195.084(9) 196.966569(5) 200.592(3)"
-    " 204.38 207.2(1) 208.98040(1) [209] [210] [222] [223] [226] [227]"
-    " 232.0377(4) 231.03588(2) 238.02891(3) [237] [244] [243] [247] [247]"
-    " [251] [252] [257] [258] [259] [266] [267] [268] [269] [270] [269]"
-    " [278] [281] [282] [285] [286] [289] [289] [293] [294] [294]"
-)
-
-
-def _get_relative_atomic_masses():
-    for mass in _relative_atomic_masses.split():
-        if mass.startswith('[') and mass.endswith(']'):
-            yield float(mass[1:-1])
-        elif '(' in mass:
-            yield float(mass.split('(')[0])
-        else:
-            yield(float(mass))
-
-relative_atomic_masses = tuple(_get_relative_atomic_masses())
-
-
-def mass_from_composition(composition):
-    """ Calculates molecular mass from atomic weights
-
-    Parameters
-    ----------
-    composition: dict
-        Dictionary mapping int (atomic number) to int (coefficient)
-
-    Returns
-    -------
-    float
-        molecular weight in atomic mass units
-
-
-    Notes
-    -----
-    Atomic number 0 denotes charge or "net electron defficiency"
-
-    Examples
-    --------
-    >>> '%.2f' % mass_from_composition({0: -1, 1: 1, 8: 1})
-    '17.01'
-    """
-    mass = 0.0
-    for k, v in composition.items():
-        if k == 0:  # electron
-            mass -= v*5.489e-4
-        else:
-            mass += v*relative_atomic_masses[k-1]
-    return mass
-
 
 def _get_charge(chgstr):
 
@@ -234,7 +142,7 @@ def _get_charge(chgstr):
             if len(before) > 0 and len(after) > 0:
                 raise ValueError("Values both before and after charge token")
             if len(before) > 0:
-                # will_be_missing_in='0.4.0'
+                # will_be_missing_in='0.8.0'
                 warnings.warn("'Fe/3+' deprecated, use e.g. 'Fe+3'",
                               ChemPyDeprecationWarning, stacklevel=3)
                 return sign * int(1 if before == '' else before)
@@ -257,8 +165,8 @@ def _formula_to_parts(formula, prefixes, suffixes):
 
     # Extract charge
     if '/' in formula:
-        # will_be_missing_in='0.4.0'
-        warnings.warn("/ depr. (before 0.4.0): use 'Fe+3' over 'Fe/3+'",
+        # will_be_missing_in='0.8.0'
+        warnings.warn("/ depr. (before 0.5.0): use 'Fe+3' over 'Fe/3+'",
                       ChemPyDeprecationWarning, stacklevel=3)
         parts = formula.split('/')
 
@@ -286,22 +194,45 @@ def _formula_to_parts(formula, prefixes, suffixes):
 def _parse_stoich(stoich):
     if stoich == 'e':  # special case, the electron is not an element
         return {}
-    return {symbols.index(k)+1: n for k, n in _parser.parseString(stoich)}
+    return {symbols.index(k)+1: n for k, n
+            in _get_formula_parser().parseString(stoich)}
 
 _greek_letters = (
     'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta', 'eta', 'theta',
-    'iota', 'kappa', 'lambda', 'mu', 'nu', 'xi', 'o', 'pi', 'rho', 'sigma',
-    'tau', 'upsilon', 'phi', 'chi', 'psi', 'omega'
+    'iota', 'kappa', 'lambda', 'mu', 'nu', 'xi', 'omicron', 'pi', 'rho',
+    'sigma', 'tau', 'upsilon', 'phi', 'chi', 'psi', 'omega'
 )
+_greek_u = u'αβγδεζηθικλμνξοπρστυφχψω'
 
 _latex_mapping = {k + '-': '\\' + k + '-' for k in _greek_letters}
 _latex_mapping['epsilon-'] = '\\varepsilon-'
+_latex_mapping['omicron-'] = 'o-'
 _latex_mapping['.'] = '^\\bullet '
+_latex_infix_mapping = {'.': '\\cdot '}
+
+_unicode_mapping = {k + '-': v + '-' for k, v in zip(_greek_letters, _greek_u)}
+_unicode_mapping['.'] = u'⋅'
+_unicode_infix_mapping = {'.': u'·'}
+
+_html_mapping = {k + '-': '&' + k + ';-' for k in _greek_letters}
+_html_mapping['.'] = '&sdot;'
+_html_infix_mapping = _html_mapping
 
 
-def to_composition(formula,
-                   prefixes=None,
-                   suffixes=('(s)', '(l)', '(g)', '(aq)')):
+def _get_leading_integer(s):
+    m = re.findall(r'^\d+', s)
+    if len(m) == 0:
+        m = 1
+    elif len(m) == 1:
+        s = s[len(m[0]):]
+        m = int(m[0])
+    else:
+        raise ValueError("Failed to parse: %s" % s)
+    return m, s
+
+
+def formula_to_composition(formula, prefixes=None,
+                           suffixes=('(s)', '(l)', '(g)', '(aq)')):
     """ Parse composition of formula representing a chemical formula
 
     Composition is represented as a dict mapping int -> int (atomic
@@ -318,19 +249,33 @@ def to_composition(formula,
 
     Examples
     --------
-    >>> to_composition('NH4+') == {0: 1, 1: 4, 7: 1}
+    >>> formula_to_composition('NH4+') == {0: 1, 1: 4, 7: 1}
     True
-    >>> to_composition('.NHO-(aq)') == {0: -1, 1: 1, 7: 1, 8: 1}
+    >>> formula_to_composition('.NHO-(aq)') == {0: -1, 1: 1, 7: 1, 8: 1}
+    True
+    >>> formula_to_composition('Na2CO3.7H2O') == {11: 2, 6: 1, 8: 10, 1: 14}
     True
 
     """
     if prefixes is None:
         prefixes = _latex_mapping.keys()
-    parts = _formula_to_parts(formula, prefixes, suffixes)
-    comp = _parse_stoich(parts[0])
-    if parts[1] is not None:
-        comp[0] = _get_charge(parts[1])
-    return comp
+    stoich_tok, chg_tok = _formula_to_parts(formula, prefixes, suffixes)[:2]
+    tot_comp = {}
+    parts = stoich_tok.split('.')
+    for idx, stoich in enumerate(parts):
+        if idx == 0:
+            m = 1
+        else:
+            m, stoich = _get_leading_integer(stoich)
+        comp = _parse_stoich(stoich)
+        for k, v in comp.items():
+            if k not in tot_comp:
+                tot_comp[k] = m*v
+            else:
+                tot_comp[k] += m*v
+    if chg_tok is not None:
+        tot_comp[0] = _get_charge(chg_tok)
+    return tot_comp
 
 
 def _subs(string, patterns):
@@ -339,53 +284,6 @@ def _subs(string, patterns):
     return string
 
 
-def to_latex(formula,
-             prefixes=None,
-             suffixes=('(s)', '(l)', '(g)', '(aq)'),
-             intercept=None):
-    r""" Convert formula string to latex representation
-
-    Parameters
-    ----------
-    formula: str
-        Chemical formula, e.g. 'H2O', 'Fe/3+', 'Cl-'
-    prefixes: dict
-        Prefix transofmrations, default: greek letters and .
-    suffixes: tuple of strings
-        Suffixes to keep, e.g. ('(g)', '(s)')
-
-    Examples
-    --------
-    >>> to_latex('NH4+')
-    'NH_{4}^{+}'
-    >>> to_latex('Fe(CN)6+2')
-    'Fe(CN)_{6}^{2+}'
-    >>> to_latex('Fe(CN)6+2(aq)')
-    'Fe(CN)_{6}^{2+}(aq)'
-    >>> to_latex('.NHO-(aq)')
-    '^\\bullet NHO^{-}(aq)'
-    >>> to_latex('alpha-FeOOH(s)')
-    '\\alpha-FeOOH(s)'
-
-    """
-    if prefixes is None:
-        prefixes = _latex_mapping
-    parts = _formula_to_parts(formula, prefixes.keys(), suffixes)
-
-    string = re.sub(r'([0-9]+)', r'_{\1}', parts[0])
-    if parts[1] is not None:
-        chg = _get_charge(parts[1])
-        if chg < 0:
-            token = '-' if chg == -1 else '%d-' % -chg
-        if chg > 0:
-            token = '+' if chg == 1 else '%d+' % chg
-        string += '^{%s}' % token
-    if len(parts) > 4:
-        raise ValueError("Incorrect formula")
-    pre_str = ''.join(map(lambda x: _subs(x, prefixes), parts[2]))
-    return pre_str + string + ''.join(parts[3])
-
-
 def _parse_multiplicity(strings, substance_keys=None):
     """
     Examples
@@ -394,14 +292,25 @@ def _parse_multiplicity(strings, substance_keys=None):
     True
     >>> _parse_multiplicity(['2 * H2O2', 'O2']) == {'H2O2': 2, 'O2': 1}
     True
+    >>> _parse_multiplicity(['']) == {}
+    True
+    >>> _parse_multiplicity(['H2O', 'H2O']) == {'H2O': 2}
+    True
 
     """
     result = {}
-    for items in [re.split(' \* | ', s) for s in strings]:
-        if len(items) == 1:
-            result[items[0]] = 1
+    for items in [re.split(' \\* | ', s) for s in strings]:
+        items = [x for x in items if x != '']
+        if len(items) == 0:
+            continue
+        elif len(items) == 1:
+            if items[0] not in result:
+                result[items[0]] = 0
+            result[items[0]] += 1
         elif len(items) == 2:
-            result[items[1]] = int(items[0])
+            if items[1] not in result:
+                result[items[1]] = 0
+            result[items[1]] += float(items[0]) if '.' in items[0] or 'e' in items[0] else int(items[0])
         else:
             raise ValueError("To many parts in substring")
     if substance_keys is not None:
@@ -411,7 +320,7 @@ def _parse_multiplicity(strings, substance_keys=None):
     return result
 
 
-def to_reaction(line, substance_keys, token, Cls, globals_=None):
+def to_reaction(line, substance_keys, token, Cls, globals_=None, **kwargs):
     """ Parses a string into a Reaction object and substances
 
     Reac1 + 2 Reac2 + (2 Reac1) -> Prod1 + Prod2; 10**3.7; ref='doi:12/ab'
@@ -438,40 +347,190 @@ def to_reaction(line, substance_keys, token, Cls, globals_=None):
     with running this on untrusted data.
 
     """
-    # TODO: add handling of units.
     if globals_ is None:
-        import chempy
-        from chempy.units import default_units
-        globals_ = {'chempy': chempy, 'default_units': default_units}
-        globals_.update(default_units.as_dict())
-
-    try:
-        stoich, param, kwargs = map(str.strip, line.rstrip('\n').split(';'))
-    except ValueError:
-        stoich, param = map(str.strip, line.rstrip('\n').split(';'))
-        kwargs = {}
+        globals_ = get_parsing_context()
+    parts = line.rstrip('\n').split(';')
+    stoich = parts[0].strip()
+    if len(parts) > 2:
+        kwargs.update(eval('dict('+';'.join(parts[2:])+'\n)', globals_ or {}))
+    if len(parts) > 1:
+        param = parts[1].strip()
     else:
-        kwargs = eval('dict('+kwargs+')', globals_)
+        param = kwargs.pop('param', 'None')
 
-    param = eval(param, globals_)
+    if isinstance(param, str):
+        if param.startswith("'") and param.endswith("'") and "'" not in param[1:-1]:
+            from ..kinetics.rates import MassAction
+            param = MassAction(unique_keys=(param[1:-1],))
+        else:
+            param = None if globals_ is False else eval(param, globals_)
 
     if token not in stoich:
         raise ValueError("Missing token: %s" % token)
 
-    reac_prod = [[y.strip() for y in x.split(' + ')] for
-                 x in stoich.split(token)]
+    reac_prod = [[y.strip() for y in x.split(' + ')] for x in stoich.split(token)]
 
     act, inact = [], []
-    for side in reac_prod:
-        if side[-1].startswith('('):
-            if not side[-1].endswith(')'):
-                raise ValueError("Bad format (missing closing paren)")
-            inact.append(_parse_multiplicity(side[-1][1:-1].split(' + '), substance_keys))
-            act.append(_parse_multiplicity(side[:-1], substance_keys))
-        else:
-            inact.append({})
-            act.append(_parse_multiplicity(side, substance_keys))
+    for elements in reac_prod:
+        act.append(_parse_multiplicity([x for x in elements if not x.startswith('(')], substance_keys))
+        inact.append(_parse_multiplicity(
+            [x[1:-1] for x in elements if x.startswith('(') and x.endswith(')')],
+            substance_keys
+        ))
 
     # stoich coeff -> dict
     return Cls(act[0], act[1], param, inact_reac=inact[0],
                inact_prod=inact[1], **kwargs)
+
+
+def _formula_to_format(sub, sup, formula, prefixes=None,
+                       infixes=None, suffixes=('(s)', '(l)', '(g)', '(aq)')):
+    parts = _formula_to_parts(formula, prefixes.keys(), suffixes)
+    stoichs = parts[0].split('.')
+    string = ''
+    for idx, stoich in enumerate(stoichs):
+        if idx == 0:
+            m = 1
+        else:
+            m, stoich = _get_leading_integer(stoich)
+            string += _subs('.', infixes)
+        if m != 1:
+            string += str(m)
+        string += re.sub(r'([0-9]+)', lambda m: sub(m.group(1)), stoich)
+
+    if parts[1] is not None:
+        chg = _get_charge(parts[1])
+        if chg < 0:
+            token = '-' if chg == -1 else '%d-' % -chg
+        if chg > 0:
+            token = '+' if chg == 1 else '%d+' % chg
+        string += sup(token)
+    if len(parts) > 4:
+        raise ValueError("Incorrect formula")
+    pre_str = ''.join(map(lambda x: _subs(x, prefixes), parts[2]))
+    return pre_str + string + ''.join(parts[3])
+
+
+def formula_to_latex(formula, prefixes=None, infixes=None, **kwargs):
+    r""" Convert formula string to latex representation
+
+    Parameters
+    ----------
+    formula: str
+        Chemical formula, e.g. 'H2O', 'Fe+3', 'Cl-'
+    prefixes: dict
+        Prefix transofmrations, default: greek letters and .
+    infixes: dict
+        Infix transformations, default: .
+    suffixes: iterable of str
+        What suffixes not to interpret, default: (s), (l), (g), (aq)
+
+    Examples
+    --------
+    >>> formula_to_latex('NH4+')
+    'NH_{4}^{+}'
+    >>> formula_to_latex('Fe(CN)6+2')
+    'Fe(CN)_{6}^{2+}'
+    >>> formula_to_latex('Fe(CN)6+2(aq)')
+    'Fe(CN)_{6}^{2+}(aq)'
+    >>> formula_to_latex('.NHO-(aq)')
+    '^\\bullet NHO^{-}(aq)'
+    >>> formula_to_latex('alpha-FeOOH(s)')
+    '\\alpha-FeOOH(s)'
+
+    """
+    if prefixes is None:
+        prefixes = _latex_mapping
+    if infixes is None:
+        infixes = _latex_infix_mapping
+    return _formula_to_format(lambda x: '_{%s}' % x, lambda x: '^{%s}' % x,
+                              formula, prefixes, infixes, **kwargs)
+
+
+_unicode_sub = {}
+
+for k, v in enumerate(u"₀₁₂₃₄₅₆₇₈₉"):
+    _unicode_sub[str(k)] = v
+
+_unicode_sup = {
+    '+': u'⁺',
+    '-': u'⁻',
+}
+
+for k, v in enumerate(u"⁰¹²³⁴⁵⁶⁷⁸⁹"):
+    _unicode_sup[str(k)] = v
+
+
+def formula_to_unicode(formula, prefixes=None, infixes=None, **kwargs):
+    u""" Convert formula string to unicode string representation
+
+    Parameters
+    ----------
+    formula : str
+        Chemical formula, e.g. 'H2O', 'Fe+3', 'Cl-'
+    prefixes : dict
+        Prefix transofmrations, default: greek letters and .
+    infixes : dict
+        Infix transofmrations, default: .
+    suffixes : tuple of strings
+        Suffixes to keep, e.g. ('(g)', '(s)')
+
+    Examples
+    --------
+    >>> formula_to_unicode('NH4+') == u'NH₄⁺'
+    True
+    >>> formula_to_unicode('Fe(CN)6+2') == u'Fe(CN)₆²⁺'
+    True
+    >>> formula_to_unicode('Fe(CN)6+2(aq)') == u'Fe(CN)₆²⁺(aq)'
+    True
+    >>> formula_to_unicode('.NHO-(aq)') == u'⋅NHO⁻(aq)'
+    True
+    >>> formula_to_unicode('alpha-FeOOH(s)') == u'α-FeOOH(s)'
+    True
+
+    """
+    if prefixes is None:
+        prefixes = _unicode_mapping
+    if infixes is None:
+        infixes = _unicode_infix_mapping
+    return _formula_to_format(
+        lambda x: ''.join(_unicode_sub[str(_)] for _ in x),
+        lambda x: ''.join(_unicode_sup[str(_)] for _ in x),
+        formula, prefixes, infixes, **kwargs)
+
+
+def formula_to_html(formula, prefixes=None, infixes=None, **kwargs):
+    u""" Convert formula string to html string representation
+
+    Parameters
+    ----------
+    formula : str
+        Chemical formula, e.g. 'H2O', 'Fe+3', 'Cl-'
+    prefixes : dict
+        Prefix transformations, default: greek letters and .
+    infixes : dict
+        Infix transformations, default: .
+    suffixes : tuple of strings
+        Suffixes to keep, e.g. ('(g)', '(s)')
+
+    Examples
+    --------
+    >>> formula_to_html('NH4+')
+    'NH<sub>4</sub><sup>+</sup>'
+    >>> formula_to_html('Fe(CN)6+2')
+    'Fe(CN)<sub>6</sub><sup>2+</sup>'
+    >>> formula_to_html('Fe(CN)6+2(aq)')
+    'Fe(CN)<sub>6</sub><sup>2+</sup>(aq)'
+    >>> formula_to_html('.NHO-(aq)')
+    '&sdot;NHO<sup>-</sup>(aq)'
+    >>> formula_to_html('alpha-FeOOH(s)')
+    '&alpha;-FeOOH(s)'
+
+    """
+    if prefixes is None:
+        prefixes = _html_mapping
+    if infixes is None:
+        infixes = _html_infix_mapping
+    return _formula_to_format(lambda x: '<sub>%s</sub>' % x,
+                              lambda x: '<sup>%s</sup>' % x,
+                              formula, prefixes, infixes, **kwargs)
